#include "core/upower.h"
#include "core/consolekitdbus.h"
#include "core/upowerdbus.h"
#include "core/device.h"

#include "core/config.h"

#include "core/application.h"
#include "core/dbuswatcher.h"

#include <QDBusConnection>

using namespace core;

UPower::UPower(QObject *parent) : QObject(parent),
    m_power(0),
    m_consolekit(0),
    m_devices(),
    m_mainDevice(0) {

    // Reconnect
    connect(app->dbusWatcher(), SIGNAL(upowerAvailable()), this, SLOT(reconnectUPower()));
    connect(app->dbusWatcher(), SIGNAL(consoleKitAvailable()), this, SLOT(reconnectConsoleKit()));

    // Disconnected
    connect(app->dbusWatcher(), SIGNAL(consoleKitUnavailable()), this, SIGNAL(consolekitDisconnected()));
    connect(app->dbusWatcher(), SIGNAL(upowerUnavailable()), this, SIGNAL(upowerDisconnected()));
    connect(app->dbusWatcher(), SIGNAL(upowerUnavailable()), this, SLOT(clearDevices()));

    reconnectConsoleKit();
    reconnectUPower();
}

UPower::~UPower() {

}

bool UPower::reconnectConsoleKit() {
    if(m_consolekit)
        delete m_consolekit;
    m_consolekit = new ConsoleKitDBus(DBUS_INTERFACE_CONSOLEKIT, DBUS_OBJECT_CONSOLEKIT, QDBusConnection::systemBus(), this);
    if(isConsoleKitPresent()) {
        connectConsoleKit();
        emit consolekitConnected();
        return true;
    }
    return false;
}

bool UPower::reconnectUPower() {
    if(m_power) {
        clearDevices();
        delete m_power;
    }
    m_power = new UPowerDBus(DBUS_INTERFACE_UPOWER, DBUS_OBJECT_UPOWER, QDBusConnection::systemBus(), this);
    if(isUPowerPresent()) {
        connectUPower();
        emit upowerConnected();
        return true;
    }
    return false;
}

void UPower::connectConsoleKit() {

}

void UPower::connectUPower() {
    connect(m_power, SIGNAL(Changed()), this, SLOT(propertiesChanged()));
    connect(m_power, SIGNAL(DeviceAdded(QDBusObjectPath)), this, SLOT(deviceAdded(QDBusObjectPath)));
    connect(m_power, SIGNAL(DeviceRemoved(QDBusObjectPath)), this, SLOT(deviceRemoved(QDBusObjectPath)));
    connect(m_power, SIGNAL(NotifyResume(QString)), this, SIGNAL(aboutToResume()));
    connect(m_power, SIGNAL(NotifySleep(QString)), this, SIGNAL(aboutToSleep()));
    discoverDevices();
}

bool UPower::restart() {
    if(isConsoleKitPresent()) {
        m_consolekit->Restart();
        return true;
    }
    return false;
}

bool UPower::stop() {
    if(isConsoleKitPresent()) {
        QDBusPendingReply<> reply = m_consolekit->Stop();
        reply.waitForFinished();
        qDebug() << reply.error();
        qDebug() << m_consolekit->lastError();
        return true;
    }
    return false;
}

bool UPower::hibernate() {
    if(canHibernate()) {
        m_power->AboutToSleep("hibernate");
        QTimer *t = new QTimer(this);
        t->setInterval(1000);
        t->setSingleShot(true);
        connect(t, SIGNAL(timeout()), this, SLOT(doHibernate()));
        return true;
    }
    return false;
}

bool UPower::suspend() {
    if(canSuspend()) {
        m_power->AboutToSleep("suspend");
        QTimer *t = new QTimer(this);
        t->setInterval(1000);
        t->setSingleShot(true);
        connect(t, SIGNAL(timeout()), this, SLOT(doSuspend()));
        return true;
    }
    return false;
}

bool UPower::canSuspend() {
    return isUPowerPresent() && m_power->SuspendAllowed();
}

bool UPower::canHibernate() {
    return isUPowerPresent() && m_power->HibernateAllowed();
}

bool UPower::isUPowerPresent() const {
    return m_power->isValid();
}

bool UPower::isConsoleKitPresent() const {
    return m_consolekit->isValid();
}

Device *UPower::device(const QString &name) const {
    if(m_devices.contains(name)) {
        return m_devices[name];
    }
    return 0;
}

bool UPower::deviceExists(const QString &name) const {
    return m_devices.contains(name);
}

void UPower::deviceAdded(QDBusObjectPath path) {
    if(m_devices.contains(path.path())) {
        qWarning("Battery already present !");
        return;
    }
    addDevice(new Device(path.path(), this));
}

void UPower::deviceRemoved(QDBusObjectPath path) {
    if(!m_devices.contains(path.path())) {
        qWarning("Battery not found. Can't be removed !");
        return;
    }
    removeDevice(path.path());
}

void UPower::addDevice(Device *d) {
    QSettings s;
    m_devices.insert(d->name(), d);
    emit deviceAdded(d->name());
    emit deviceAdded(d);
    if(s.value("MainDevice", "") == d->name()) {
        setMainDevice(d);
    }
}

void UPower::removeDevice(const QString &path) {
    m_devices[path]->deleteLater();
    m_devices.remove(path);
    emit deviceRemoved(path);
}

void UPower::setMainDevice(Device *d) {
    if(d != m_mainDevice) {
        core::Device *old = m_mainDevice;
        m_mainDevice = d;
        emit mainDeviceChanged(d);
        emit mainDeviceChanged(old, d);

        QSettings s;
        if(d) {
            s.setValue("MainDevice", d->name());
        } else {
            s.setValue("MainDevice", "None");
        }
    }
}

void UPower::removeDevice(Device *d) {
    if(!m_devices.contains(d->name())) {
        qWarning("Battery not found. Can't be removed !");
        return ;
    }
    removeDevice(m_devices[d->name()]);
    d->deleteLater();
}

void UPower::discoverDevices() {
    clearDevices();
    if(isUPowerPresent()) {
       QDBusPendingReply<QList<QDBusObjectPath> > reply = m_power->EnumerateDevices();
       QDBusPendingCallWatcher *w = new QDBusPendingCallWatcher(reply);
       connect(w, SIGNAL(finished(QDBusPendingCallWatcher*)), this, SLOT(discoveringCompleted(QDBusPendingCallWatcher*)));
    }
}

void UPower::clearDevices() {
    foreach(Device *d, m_devices.values()) {
        removeDevice(d);
    }
    m_devices.clear();
}

void UPower::discoveringCompleted(QDBusPendingCallWatcher *call) {
    QDBusPendingReply<QList<QDBusObjectPath> > reply = *call;
    if(!reply.isError() && reply.isFinished() && reply.isValid()) {
        QList<QDBusObjectPath> list =reply.value();
        foreach(QDBusObjectPath path, list) {
            deviceAdded(path);
        }
    }
    call->deleteLater();
}

void UPower::doHibernate() {
    if(canHibernate())
        m_power->Hibernate();
    if(sender())
        sender()->deleteLater();
}

void UPower::doSuspend() {
    if(canSuspend())
        m_power->Suspend();
    if(sender())
        sender()->deleteLater();
}

void UPower::propertiesChanged() {

}

QString UPower::nameFromPath(const QDBusObjectPath &path) {
    return nameFromPath(path.path());
}

QString UPower::nameFromPath(const QString &path) {
    QFileInfo info(path);
    return info.baseName();
}
