#include "mainwindow.h"
#include "ui_mainwindow.h"

#include "core/device.h"
#include "core/upower.h"

#include "gui/devicecombobox.h"
#include "gui/devicewidget.h"
#include "gui/devicestackedwidget.h"
#include "gui/systemtray.h"

#include "core/application.h"

#include <QMessageBox>

using namespace gui;

MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow)
{
    ui->setupUi(this);

    QSettings s;
    s.beginGroup("Settings");
    bool batteryOnly = s.value("ShowOnlyBatteries", false).toBool();
    s.endGroup();

    ui->deviceComboBox->setOnlyBattery(batteryOnly);

    connect(app->upower(), SIGNAL(mainDeviceChanged(core::Device*,core::Device*)), this, SLOT(setMainDevice(core::Device*,core::Device*)));
    connect(ui->deviceComboBox, SIGNAL(currentDeviceChanged(core::Device*)), ui->deviceStack, SLOT(setCurrentDevice(core::Device*)));
    connect(ui->deviceComboBox, SIGNAL(noDevices()), ui->deviceStack, SLOT(noDevices()));
    connect(ui->action_Quit, SIGNAL(triggered()), app, SLOT(quit()));
    connect(ui->settingsPage, SIGNAL(onlyBatteries(bool)), ui->deviceComboBox, SLOT(setOnlyBattery(bool)));

    connect(app->upower(), SIGNAL(upowerConnected()), this, SLOT(upowerConnected()));
    connect(app->upower(), SIGNAL(upowerDisconnected()), this, SLOT(upowerDisconnected()));

    connect(ui->action_About_Qt, SIGNAL(triggered()), app, SLOT(aboutQt()));
    connect(ui->actionAbout, SIGNAL(triggered()), this, SLOT(about()));

    ui->batteriesPage->setEnabled(app->upower()->isUPowerPresent());

    ui->stackedWidget->setCurrentIndex(0);

    if(ui->deviceComboBox->isEmpty()) {
        ui->deviceStack->setCurrentIndex(0);
    }

    if(!s.value("Settings/StartMinimized", false).toBool()) {
        show();
        s.beginGroup("MainWindow");
        resize(s.value("Size").toSize());
        move(s.value("Position").toPoint());
        s.endGroup();
    }
}

MainWindow::~MainWindow()
{
    delete ui;
}

void MainWindow::closeEvent(QCloseEvent*) {
    QSettings s;
    s.beginGroup("MainWindow");
    s.setValue("Size", size());
    s.setValue("Position", pos());
    s.endGroup();
}

void MainWindow::about() {
    QMessageBox::about(this,
                       "About BatterySystemTray",
                       QString(
                       "<b>Battery Information and Cpu "
                       "frequency management Application</b>"
                       "<br/>"
                        "<b>Version:</b> %1<br/>"
                       "<br/>Under GPLv3 License<br/>"
                       "<br/>Project : "
                       "<a href=\"http://pt.pyrotools.org/\">"
                       "http://www.pyrotools.org/</a>"
                       "<br/>"
                       "Brieuc Roblin "
                       "(<a href=\"mailto:brieuc.roblin@gmail.com\">"
                       "brieuc.roblin@gmail.com"
                       "</a>)"
                       "").arg(app->applicationVersion()));
}


void MainWindow::setMainDevice(core::Device *o, core::Device *n) {
    if(o) {
        disconnect(o, SIGNAL(changed()), this, SLOT(updateMainDevice()));
    }
    if(n) {
        setWindowIcon(n->icon());
        ui->action_Devices->setIcon(n->icon());
        connect(n, SIGNAL(changed()), this, SLOT(updateMainDevice()));
    } else {
        setWindowIcon(QIcon(":/ui/upower"));
        ui->action_Devices->setIcon(QIcon(":/ui/battery"));
    }
}

void MainWindow::updateMainDevice() {
    core::Device *d = qobject_cast<core::Device*>(sender());
    if(d) {
        setWindowIcon(d->icon());
        ui->action_Devices->setIcon(d->icon());
    }
}

void MainWindow::on_action_UCpufreq_triggered() {
    ui->stackedWidget->setCurrentIndex(1);
}

void MainWindow::on_action_Devices_triggered() {
    ui->stackedWidget->setCurrentIndex(0);
    if(app->upower()->mainDevice()) {
        app->upower()->mainDevice()->refresh();
    }
}

void MainWindow::on_action_Settings_triggered() {
    ui->stackedWidget->setCurrentIndex(2);
}

void MainWindow::display() {
    QSettings s;
    s.beginGroup("MainWindow");
    if(isVisible()) {
        s.setValue("Size", size());
        s.setValue("Position", pos());
    } else {
        resize(s.value("Size").toSize());
        move(s.value("Position").toPoint());
    }
    setVisible(!isVisible());

    s.endGroup();
}

void MainWindow::upowerConnected() {
    ui->batteriesPage->setEnabled(true);
}

void MainWindow::upowerDisconnected() {
    ui->batteriesPage->setEnabled(false);
}


