#include "systemtray.h"

#include "core/application.h"
#include "core/device.h"
#include "core/upower.h"
#include "core/ucpufreq.h"
#include "core/ucpufreqdbus.h"

#include <QAction>
#include <QMenu>

using namespace gui;

SystemTray::SystemTray(QObject *parent) : QSystemTrayIcon(parent) {
    connect(this, SIGNAL(activated(QSystemTrayIcon::ActivationReason)), this, SLOT(iconActivated(QSystemTrayIcon::ActivationReason)));

    setIcon(QIcon(":/ui/upower"));

    createActions();
    createIcons();
    show();

    connect(app->upower(), SIGNAL(mainDeviceChanged(core::Device*,core::Device*)), this, SLOT(setMainDevice(core::Device*,core::Device*)));
    connect(app->upower(), SIGNAL(deviceAdded(core::Device*)), this, SLOT(connectDevice(core::Device*)));
    connect(app->cpufreq(), SIGNAL(connected()), this, SLOT(ucpufreqConnected()));
    connect(app->cpufreq(), SIGNAL(disconnected()), this, SLOT(ucpufreqDisconnected()));
}

SystemTray::~SystemTray() {
    delete m_menu;
}

void SystemTray::createActions() {
    m_minimizeAction = new QAction(QIcon(":/ui/minimize"), tr("Minimize"), this);
    connect(m_minimizeAction, SIGNAL(triggered()), this, SIGNAL(minimizeClicked()));

    m_restoreAction = new QAction(QIcon(":/ui/restore"), tr("Restore"), this);
    connect(m_restoreAction, SIGNAL(triggered()), this, SIGNAL(maximizeClicked()));

    m_quitAction = new QAction(QIcon(":/ui/exit"), tr("Exit"), this);
    connect(m_quitAction, SIGNAL(triggered()), app, SLOT(quit()));

    m_s2ram = new QAction(QIcon(":/ui/s2ram"), tr("Suspend"), this);
    connect(m_s2ram, SIGNAL(triggered()), app->upower(), SLOT(suspend()));

    m_s2disk = new QAction(QIcon(":/ui/s2disk"), tr("Hibernate"), this);
    connect(m_s2disk, SIGNAL(triggered()), app->upower(), SLOT(hibernate()));

    m_reboot = new QAction(QIcon(":/ui/reboot"), tr("Reboot"), this);
    connect(m_reboot, SIGNAL(triggered()), app->upower(), SLOT(restart()));

    m_halt = new QAction(QIcon(":/ui/halt"), tr("Halt"), this);
    connect(m_halt, SIGNAL(triggered()), app->upower(), SLOT(stop()));

}

void SystemTray::createIcons() {
    m_menu = new QMenu(tr("BatterySystemTray Menu"));

    m_ucpufreq_menu = new QMenu(tr("UCpufreq"), m_menu);
    m_ucpufreq_menu->setIcon(QIcon(":/ui/cpu"));
    m_ucpufreq_menu->setEnabled(app->cpufreq()->isConnected());
    // connect(m_ucpufreq_menu, SIGNAL(aboutToShow()), this, SLOT(updateUCpuFreqValues()));

    m_governors_menu = new QMenu(tr("Governor"), m_ucpufreq_menu);
    m_governors_menu->setIcon(QIcon(":/ui/governor"));
    m_frequencies_menu = new QMenu(tr("Frequency"), m_ucpufreq_menu);
    m_frequencies_menu->setIcon(QIcon(":/ui/frequency"));

    m_frequenciesActionGroup = new QActionGroup(m_frequencies_menu);
    m_governorsActionGroup = new QActionGroup(m_governors_menu);

    QList<qulonglong> frequencies = app->cpufreq()->interface()->frequencies();

    // Convert qulonglong to Human readable string
    foreach(qulonglong f, frequencies) {
        QAction *a = m_frequencies_menu->addAction(frequencyToString(f));
        a->setCheckable(true);
        a->setActionGroup(m_frequenciesActionGroup);
        a->setProperty("Frequency", f);
    }

    m_governorsList = app->cpufreq()->interface()->governors();

    foreach(QString governor, m_governorsList) {
        QAction *a = m_governors_menu->addAction(governor);
        a->setCheckable(true);
        a->setActionGroup(m_governorsActionGroup);
    }

    connect(m_governors_menu, SIGNAL(aboutToShow()), this, SLOT(updateGovernors()));
    connect(m_frequencies_menu, SIGNAL(aboutToShow()), this, SLOT(updateFrequencies()));
    connect(m_governorsActionGroup, SIGNAL(selected(QAction*)), this, SLOT(changeGovernor(QAction*)));
    connect(m_frequenciesActionGroup, SIGNAL(selected(QAction*)), this, SLOT(changeFrequency(QAction*)));

    m_ucpufreq_menu->addMenu(m_governors_menu);
    m_ucpufreq_menu->addMenu(m_frequencies_menu);

    m_menu->addSeparator();
    m_menu->addAction(m_halt);
    m_menu->addAction(m_reboot);
    m_menu->addSeparator();
    m_menu->addAction(m_s2disk);
    m_menu->addAction(m_s2ram);
    m_menu->addSeparator();
    m_menu->addAction(m_minimizeAction);
    m_menu->addAction(m_restoreAction);
    m_menu->addSeparator();
    m_menu->addMenu(m_ucpufreq_menu);
    m_menu->addSeparator();
    m_menu->addAction(m_quitAction);

    setContextMenu(m_menu);
}

void SystemTray::iconActivated( QSystemTrayIcon::ActivationReason reason ) {
    switch(reason) {
        case QSystemTrayIcon::Trigger:
            emit showClicked();
            break;
        default:
            break;
    }
}

bool SystemTray::event(QEvent *event) {
    if(event->type() == QEvent::ToolTip) {
        core::Device *d = app->upower()->mainDevice();
        QString title, msg;
        if(d) {
            title = d->prettyName();
            switch(d->interface().type()) {
                case core::Device::Battery:
                    msg = QString("%1"). number(d->interface().percentage(), 'f', 0) + " %";
                    msg += d->timeLeft().toString(" ( hh:mm:ss )");
                    break;
                case core::Device::LinePower:
                    if(d->interface().online()) {
                        msg = "Online";
                    } else {
                        msg = "Offline";
                    }
                    break;
                default:
                    msg = "No information...";
                    break;
            }
        } else {
            title = "No main device";
            msg = "Please select a main device or plug the device";
        }
        showMessage(title, msg, Information, 2000);
        event->accept();
        return true;
    }
    return false;
}

void SystemTray::setMainDevice(core::Device *o, core::Device *n) {
    if(o) {
        disconnect(o, SIGNAL(changed()), this, SLOT(mainDeviceChanged()));
    }
    if(n) {
        setIcon(n->icon());
        connect(n, SIGNAL(changed()), this, SLOT(mainDeviceChanged()));
    } else {
        setIcon(QIcon(":/ui/upower"));
    }

}

void SystemTray::connectDevice(core::Device *d) {
    connect(d, SIGNAL(online()), this, SLOT(deviceOnline()));
    connect(d, SIGNAL(offline()), this, SLOT(deviceOffline()));
    connect(d, SIGNAL(low()), this, SLOT(deviceLow()));
    connect(d, SIGNAL(full()), this, SLOT(deviceFull()));
}

void SystemTray::ucpufreqConnected() {
    m_ucpufreq_menu->setEnabled(true);
}

void SystemTray::ucpufreqDisconnected() {
    m_ucpufreq_menu->setEnabled(false);
}

void SystemTray::updateGovernors() {
    QStringList governors = app->cpufreq()->interface()->governors();

    foreach(QString g, governors) {
        if(!m_governorsList.contains(g)) {
            QAction *a = m_governors_menu->addAction(g);
            a->setCheckable(true);
            a->setActionGroup(m_governorsActionGroup);
        }
    }

    foreach(QAction *a, m_governorsActionGroup->actions()) {
        if(a->text() == app->cpufreq()->governor()) {
            a->setChecked(true);
        } else {
            a->setChecked(false);
        }
    }
}

void SystemTray::updateFrequencies() {
    foreach(QAction *a, m_frequenciesActionGroup->actions()) {
        if(a->property("Frequency").toULongLong() == app->cpufreq()->frequency()) {
            a->setChecked(true);
        } else {
            a->setChecked(false);
        }
    }
}

void SystemTray::deviceOffline() {
    core::Device *d = qobject_cast<core::Device*>(sender());
    if(d) {
        showMessage(d->prettyName(), "Offline", Warning);
    }
}

void SystemTray::deviceOnline() {
    core::Device *d = qobject_cast<core::Device*>(sender());
    if(d) {
        showMessage(d->prettyName(), "Online", Information);
    }
}

void SystemTray::deviceLow() {
    core::Device *d = qobject_cast<core::Device*>(sender());
    if(d) {
        showMessage(d->prettyName(), "Low on battery", Warning);
    }
}

void SystemTray::deviceFull() {
    core::Device *d = qobject_cast<core::Device*>(sender());
    if(d) {
        showMessage(d->prettyName(), "Is fully charged", Information);
    }
}

void SystemTray::mainDeviceChanged() {
    core::Device *d = qobject_cast<core::Device*>(sender());
    if(d) {
        setIcon(d->icon());
    }

}

void SystemTray::changeGovernor(QAction *a) {
    QString governor = a->text();
    app->cpufreq()->setGovernor(governor);
}

void SystemTray::changeFrequency(QAction *a) {
    qulonglong frequency = a->property("Frequency").toULongLong();
    app->cpufreq()->setFrequency(frequency);
}

QString SystemTray::frequencyToString(qulonglong f) {
   if(f > ( 1000 * 1000)) {
       return QString("%1 Ghz").arg((double) f/1000/1000);
   } else if( f > 1000 ) {
       return QString("%1 Mhz").arg((double) f/1000);
   } else {
       return QString("%1").arg(f);
   }
}
